//
//  LXCircleAnimationView.m
//  LXCircleAnimationView
//
//  Created by Leexin on 15/12/18.
//  Copyright © 2015年 Garden.Lee. All rights reserved.
//

#import "LXCircleAnimationView.h"
#import "UIView+Extensions.h"

#import "CACustomTextLayer.h"

#define degreesToRadians(x) (M_PI*(x)/180.0) //把角度转换成PI的方式
static const CGFloat kMarkerRadius = 5.f; // 光标直径

static const CGFloat kTimerInterval = 0.03f;

@interface LXCircleAnimationView ()

@property (nonatomic, strong) CAShapeLayer *bottomLayer; // 进度条底色
@property (nonatomic, assign) CGFloat lineWidth; // 弧线宽度

@property (nonatomic, strong) UIImageView *markerImageView; // 光标

@property (nonatomic, strong) UIImageView *bgImageView; // 背景图片

@property (nonatomic, assign) CGFloat circelRadius; //圆直径
@property (nonatomic, assign) CGFloat stareAngle; // 开始角度
@property (nonatomic, assign) CGFloat endAngle; // 结束角度

@property (nonatomic, assign) CGFloat totalAnimationTime;

@property (nonatomic, strong) CACustomTextLayer * jumpNOLayer;

@end

@implementation LXCircleAnimationView

#pragma mark - Life cycle

- (instancetype)initWithFrame:(CGRect)frame {
    
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = [UIColor clearColor];
        
        self.circelRadius = self.frame.size.width - 10.f;
        self.lineWidth = 2.f;
        self.stareAngle = -200.f;
        self.endAngle = 20.f;
        
        // 尺寸需根据图片进行调整
        self.bgImageView.frame = CGRectMake(6, 6, self.circelRadius, self.circelRadius * 2 / 3);
        self.bgImageView.backgroundColor = [UIColor clearColor];
        [self addSubview:self.bgImageView];
        
        [self initSubView];
        
        [self setupMarkerImageView];
        
        [self setupJumpNOView];
    }
    return self;
}

- (void)initSubView {
    
    // 圆形路径
    UIBezierPath *path = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.width / 2, self.height / 2)
                                                        radius:(self.circelRadius - self.lineWidth) / 2
                                                    startAngle:degreesToRadians(self.stareAngle)
                                                      endAngle:degreesToRadians(self.endAngle)
                                                     clockwise:YES];
    
    // 底色
    self.bottomLayer = [CAShapeLayer layer];
    self.bottomLayer.frame = self.bounds;
    self.bottomLayer.fillColor = [[UIColor clearColor] CGColor];
    self.bottomLayer.strokeColor = [[UIColor  colorWithRed:206.f / 256.f green:241.f / 256.f blue:227.f alpha:1.f] CGColor];
    self.bottomLayer.opacity = 0.5;
    self.bottomLayer.lineCap = kCALineCapRound;
    self.bottomLayer.lineWidth = self.lineWidth;
    self.bottomLayer.path = [path CGPath];
    [self.layer addSublayer:self.bottomLayer];
    
    // 240 是用整个弧度的角度之和 |-200| + 20 = 220
    [self createAnimationWithStartAngle:degreesToRadians(self.stareAngle)
                               endAngle:degreesToRadians(self.stareAngle + 220 * 0)];
}

- (void)setupMarkerImageView {
    
    self.markerImageView.frame = CGRectMake(-100, self.height, kMarkerRadius, kMarkerRadius);
    self.markerImageView.layer.cornerRadius = self.markerImageView.frame.size.height / 2;
    
    [self addSubview:self.markerImageView];
}

- (void)setupJumpNOView {
    
    CACustomTextLayer * textLayer = [[CACustomTextLayer alloc] init];
    CGFloat xPixel = self.bgImageView.left + self.circelRadius / 4;
    CGFloat yPixel = self.circelRadius / 3;
    CGFloat width = self.circelRadius / 2;
    CGFloat height = self.circelRadius / 2;
    CGRect layerFrame = CGRectMake(xPixel, yPixel, width, height);
    textLayer.frame = layerFrame;
    
    textLayer.foregroundColor = [UIColor whiteColor].CGColor;
    textLayer.backgroundColor = [UIColor clearColor].CGColor;
    
    textLayer.fontSize = 60;
    
    textLayer.wrapped = YES;
    textLayer.alignmentMode = kCAAlignmentCenter;
    
    [self.layer addSublayer:textLayer];
    
    self.jumpNOLayer = textLayer;
    
    [self.jumpNOLayer jumpNumberWithDuration:0.1 fromNumber:0 toNumber:0];
}


#pragma mark - Animation

- (void)createAnimationWithStartAngle:(CGFloat)startAngle endAngle:(CGFloat)endAngle { // 光标动画
    
    // 设置动画属性
    CAKeyframeAnimation *pathAnimation = [CAKeyframeAnimation animationWithKeyPath:@"position"];
    pathAnimation.calculationMode = kCAAnimationPaced;
    pathAnimation.fillMode = kCAFillModeForwards;
    pathAnimation.removedOnCompletion = NO;
    pathAnimation.duration = self.totalAnimationTime;
    pathAnimation.timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseOut];
    pathAnimation.repeatCount = 1;
    
    // 设置动画路径
    CGMutablePathRef path = CGPathCreateMutable();
    CGPathAddArc(path, NULL, self.width / 2, self.height / 2, (self.circelRadius - kMarkerRadius / 2) / 2, startAngle, endAngle, 0);
    pathAnimation.path = path;
    CGPathRelease(path);
    
    [self.markerImageView.layer addAnimation:pathAnimation forKey:@"moveMarker"];
}

#pragma mark - Setters / Getters

- (void)setPercent:(CGFloat)percent {
    
    [self setPercent:percent animated:YES];
}

- (void)setPercent:(CGFloat)percent animated:(BOOL)animated {
    
    _percent = percent;
    
    self.totalAnimationTime = kTimerInterval * percent;
    
    [self createAnimationWithStartAngle:degreesToRadians(self.stareAngle)
                               endAngle:degreesToRadians(self.stareAngle + 220 * percent / 100)];
}

- (void)setBgImage:(UIImage *)bgImage {
    
    _bgImage = bgImage;
    self.bgImageView.image = bgImage;
}


- (UIImageView *)markerImageView {
    
    if (nil == _markerImageView) {
        _markerImageView = [[UIImageView alloc] init];
        _markerImageView.backgroundColor = [UIColor colorWithWhite:1.0 alpha:0.7];
        _markerImageView.layer.shadowColor = [UIColor whiteColor].CGColor;
        _markerImageView.layer.shadowOffset = CGSizeMake(0, 0);
        _markerImageView.layer.shadowRadius = kMarkerRadius*0.5;
        _markerImageView.layer.shadowOpacity = 1;
        _markerImageView.layer.masksToBounds = NO;
    }
    return _markerImageView;
}

- (UIImageView *)bgImageView {
    
    if (nil == _bgImageView) {
        _bgImageView = [[UIImageView alloc] init];
    }
    return _bgImageView;
}

- (void)refreshJumpNOFromNO:(NSString *)startNO toNO:(NSString *)toNO {
    
    NSLog(@"%f",self.totalAnimationTime);
    NSLog(@"%@--------%@",startNO,toNO);
    [_jumpNOLayer jumpNumberWithDuration:self.totalAnimationTime fromNumber:startNO toNumber:toNO];
}

@end
